/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_REMINDWAYPOINTSTRUCT_H
#define DPC_IMPL_TYPE_REMINDWAYPOINTSTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_uint32.h"
#include "dpc/impl_type_vector_uint8_t.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_wpextendlanestruct.h"

namespace dpc {
struct RemindWayPointStruct {
    ::UInt8 reminder_waypoint_target_road_type{ 0U };
    ::UInt32 reminder_waypoint_id{ 0U };
    ::dpc::vector_uint8_t reminder_waypoint_lane_id_list;
    ::Float reminder_waypoint_offset{ 0.0F };
    ::dpc::vector_WPExtendLaneStruct reminder_wp_extendlane_list;
    ::UInt8 reminder_waypoint_speed_limit{ 0U };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(reminder_waypoint_target_road_type);
        fun(reminder_waypoint_id);
        fun(reminder_waypoint_lane_id_list);
        fun(reminder_waypoint_offset);
        fun(reminder_wp_extendlane_list);
        fun(reminder_waypoint_speed_limit);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(reminder_waypoint_target_road_type);
        fun(reminder_waypoint_id);
        fun(reminder_waypoint_lane_id_list);
        fun(reminder_waypoint_offset);
        fun(reminder_wp_extendlane_list);
        fun(reminder_waypoint_speed_limit);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("reminder_waypoint_target_road_type", reminder_waypoint_target_road_type);
        fun("reminder_waypoint_id", reminder_waypoint_id);
        fun("reminder_waypoint_lane_id_list", reminder_waypoint_lane_id_list);
        fun("reminder_waypoint_offset", reminder_waypoint_offset);
        fun("reminder_wp_extendlane_list", reminder_wp_extendlane_list);
        fun("reminder_waypoint_speed_limit", reminder_waypoint_speed_limit);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("reminder_waypoint_target_road_type", reminder_waypoint_target_road_type);
        fun("reminder_waypoint_id", reminder_waypoint_id);
        fun("reminder_waypoint_lane_id_list", reminder_waypoint_lane_id_list);
        fun("reminder_waypoint_offset", reminder_waypoint_offset);
        fun("reminder_wp_extendlane_list", reminder_wp_extendlane_list);
        fun("reminder_waypoint_speed_limit", reminder_waypoint_speed_limit);
    }

    friend bool operator==(const ::dpc::RemindWayPointStruct& lhs, const ::dpc::RemindWayPointStruct& rhs) noexcept
    {
        return (lhs.reminder_waypoint_target_road_type == rhs.reminder_waypoint_target_road_type) && (lhs.reminder_waypoint_id == rhs.reminder_waypoint_id) && (lhs.reminder_waypoint_lane_id_list == rhs.reminder_waypoint_lane_id_list) && (fabs(static_cast<double>(lhs.reminder_waypoint_offset - rhs.reminder_waypoint_offset)) < DBL_EPSILON) && (lhs.reminder_wp_extendlane_list == rhs.reminder_wp_extendlane_list) && (lhs.reminder_waypoint_speed_limit == rhs.reminder_waypoint_speed_limit);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_REMINDWAYPOINTSTRUCT_H
